/*******************************************************************************************************************
	Corrupt a database using dbcc writepage

	Useful Links:
	https://www.sqlskills.com/blogs/paul/dbcc-writepage/
	https://www.sqlskills.com/blogs/paul/corruption-recovery-using-dbcc-writepage/
********************************************************************************************************************/

USE master;
GO

IF DATABASEPROPERTY('UhOh', 'Version') >= 0
    DROP DATABASE UhOh;
GO

CREATE DATABASE UhOh;
GO

USE UhOh;
GO

CREATE TABLE tblWhoops(
	ID INT IDENTITY(1,1) PRIMARY KEY,
	[Name] VARCHAR(200) DEFAULT 'Data'
);

INSERT INTO tblWhoops DEFAULT VALUES;
INSERT INTO tblWhoops DEFAULT VALUES;
INSERT INTO tblWhoops DEFAULT VALUES;
INSERT INTO tblWhoops DEFAULT VALUES;
INSERT INTO tblWhoops DEFAULT VALUES;
GO

CREATE NONCLUSTERED INDEX ix_tblWhoops ON tblWhoops([Name]);

SELECT * FROM dbo.tblWhoops;

/*****************************************************************************************
	
	Time to corrupt the database!
		
******************************************************************************************/

--Use this to identify clustered index details
--Get the PagePID where PageType = 1 (for clustered index and 2 for nonclustered index)
DBCC IND (UhOh, 'tblWhoops', 1); --clustered index
DBCC IND (UhOh, 'tblWhoops', 2); --nonclustered index

--Another way to get details about nonclustered index
select allocated_page_page_id, next_page_page_id, previous_page_page_id, page_level
from sys.dm_db_database_page_allocations(db_id(),object_id('tblWhoops'),2,null,'DETAILED')
where page_type_desc is not null and page_type_desc = 'INDEX_PAGE' --and page_level = 1 

--Check out database page dump
--dbcc page ( {dbname | dbid}, filenum, pagenum [, printopt={0|1|2|3} ])
DBCC PAGE(UhOh, 1, 328 /* from above query*/, 1) WITH TABLERESULTS;

/*
	Use the pageid from above to parameterize 
	dbcc writepage ({'dbname' | dbid}, fileid, pageid, offset, length, data [, directORbufferpool])
*/

ALTER DATABASE UhOh SET SINGLE_USER WITH ROLLBACK IMMEDIATE; --DBCC WRITEPAGE requires single user mode
DBCC WRITEPAGE('UhOh', 1, 328 /* get this value from sys.dm_db_database_page_allocations */, 60, 1, 0x00, 1);

SELECT * FROM dbo.tblWhoops; --what will happen?

--Examine SQL Error Log

--CHECKDB
DBCC CHECKDB('UhOh') WITH no_infomsgs;

--what database and objects are affected?
SELECT DB_ID(), DB_NAME(); --helpful with SQL Error Log to determine which databases are impacted
SELECT OBJECT_NAME(565577053);

--look for index level 2 (nonclustered)
SELECT * FROM sys.indexes WHERE [object_id] = '565577053';

--Try dropping the nonclustered index
DROP INDEX ix_tblWhoops ON tblWhoops;

--query the table again
SELECT * FROM UhOh.dbo.tblWhoops; 

ALTER DATABASE UhOh SET MULTI_USER;

--========================================================================================================

--cleanup
DROP TABLE dbo.tblWhoops;
DROP DATABASE UhOh;
